"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var Rx_1 = __importDefault(require("rxjs/Rx"));
var constants_1 = require("../constants");
var helpers_1 = require("../helpers/helpers");
var compiler_1 = require("../compiler/compiler");
var build = function (config) {
    var analysis = config.analysis, configFilePath = config.configFilePath, watch = config.watch, entryFilePath = config.entryFilePath;
    var compiler = compiler_1.initCompiler({
        configFilePath: configFilePath,
        analysis: analysis,
        webpackEnv: 'production',
        entryFilePath: entryFilePath,
    }).compiler;
    if (watch) {
        return Rx_1.default.Observable.create(function (observer) {
            compiler.hooks.done.tap('invalid', function () {
                observer.next({
                    type: constants_1.TASK_STATUS.CHANGE_START,
                });
            });
            compiler.watch({
                aggregateTimeout: 300,
                poll: false,
            }, function (err, stats) {
                if (err || stats.hasErrors()) {
                    observer.next({
                        type: constants_1.TASK_STATUS.CHANGE_ERROR,
                        payload: helpers_1.toErrorOutputString(stats),
                    });
                }
                else {
                    observer.next({
                        type: constants_1.TASK_STATUS.CHANGE_COMPLETE,
                    });
                }
            });
        });
    }
    return new Promise(function (resolve, reject) {
        return compiler.run(function (err, stats) {
            if (err) {
                return reject(err);
            }
            var hasError = stats.hasErrors();
            // TODO: 是否需要特殊处理？
            // const hasWarning = stats.hasWarnings()
            if (hasError) {
                return reject(helpers_1.toErrorOutputString(stats));
            }
            return resolve(helpers_1.toOutputString(stats));
        });
    });
};
exports.default = build;
//# sourceMappingURL=build.js.map