"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = __importDefault(require("lodash"));
var path_1 = __importDefault(require("path"));
var fs_1 = __importDefault(require("fs"));
var print_1 = __importDefault(require("./print"));
exports.print = print_1.default;
exports.formatTime = function (time) {
    return time.toTimeString().replace(/.*(\d{2}:\d{2}:\d{2}).*/, '$1');
};
exports.capFirstLetter = function (word) {
    return word
        .split('')
        .map(function (w, i) { return (i === 0 ? w.toUpperCase() : w); })
        .join('');
};
exports.isDepInstalled = function (dep) {
    try {
        require.resolve(resolvePathInProject(path_1.default.join('node_modules', dep)));
        return true;
    }
    catch (error) {
        return false;
    }
};
exports.fileExists = function (relPath) {
    try {
        require.resolve(resolvePathInProject(relPath));
        return true;
    }
    catch (error) {
        return false;
    }
};
function resolvePathInProject() {
    var relativePaths = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        relativePaths[_i] = arguments[_i];
    }
    // Make sure any symlinks in the project folder are resolved:
    // https://github.com/facebookincubator/create-react-app/issues/637
    var projectDir = fs_1.default.realpathSync(process.cwd());
    return path_1.default.resolve.apply(path_1.default, [projectDir].concat(relativePaths));
}
exports.resolvePathInProject = resolvePathInProject;
// merge objects with array without mutation
exports.mergeAnything = function (object, sources) {
    if (Array.isArray(sources)) {
        return object.concat(sources);
    }
    var object2 = lodash_1.default.cloneDeep(object);
    return lodash_1.default.mergeWith(object2, sources, function (objValue, srcValue) {
        if (lodash_1.default.isArray(objValue)) {
            return objValue.concat(srcValue);
        }
    });
};
exports.copyFileWithExistenceCheck = function (src, target) {
    if (!fs_1.default.existsSync(target)) {
        fs_1.default.copyFileSync(src, target);
        return true;
    }
    print_1.default.warn(target + " exists!");
    return false;
};
exports.getFirstExistingFile = function (chain) {
    for (var _i = 0, chain_1 = chain; _i < chain_1.length; _i++) {
        var file = chain_1[_i];
        if (exports.fileExists(file)) {
            return file;
        }
    }
};
exports.getFilename = function (absFilePath) {
    return path_1.default.basename(absFilePath);
};
//# sourceMappingURL=index.js.map