"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var webpack_1 = __importDefault(require("webpack"));
var fs_1 = __importDefault(require("fs"));
var is_wsl_1 = __importDefault(require("is-wsl"));
var terser_webpack_plugin_1 = __importDefault(require("terser-webpack-plugin"));
var webpack_bundle_analyzer_1 = require("webpack-bundle-analyzer");
var case_sensitive_paths_webpack_plugin_1 = __importDefault(require("case-sensitive-paths-webpack-plugin"));
var webpack_manifest_plugin_1 = __importDefault(require("webpack-manifest-plugin"));
var optimize_css_assets_webpack_plugin_1 = __importDefault(require("optimize-css-assets-webpack-plugin"));
var postcss_safe_parser_1 = __importDefault(require("postcss-safe-parser"));
var html_webpack_plugin_1 = __importDefault(require("html-webpack-plugin"));
var mini_css_extract_plugin_1 = __importDefault(require("mini-css-extract-plugin"));
var utils_1 = require("./utils");
var getCacheIdentifier_1 = __importDefault(require("react-dev-utils/getCacheIdentifier"));
var getCSSModuleLocalIdent_1 = __importDefault(require("react-dev-utils/getCSSModuleLocalIdent"));
var ForkTsCheckerWebpackPlugin_1 = __importDefault(require("react-dev-utils/ForkTsCheckerWebpackPlugin"));
var typescriptFormatter_1 = __importDefault(require("react-dev-utils/typescriptFormatter"));
var WatchMissingNodeModulesPlugin_1 = __importDefault(require("react-dev-utils/WatchMissingNodeModulesPlugin"));
var paths_1 = __importStar(require("./config/paths"));
var postcss_normalize_1 = __importDefault(require("postcss-normalize"));
var resolve_1 = __importDefault(require("resolve"));
var assetsFolder = paths_1.folders.assets;
// TODO: 能否获取到环境变量
var shouldUseSourceMap = process.env.GENERATE_SOURCEMAP !== 'false';
var moduleFileExtensions = [
    'web.mjs',
    'mjs',
    'web.js',
    'js',
    'web.ts',
    'ts',
    'web.tsx',
    'tsx',
    'json',
    'web.jsx',
    'jsx',
];
var defaultOptions = {
    analysis: false,
    servedPath: '/',
};
// Check if TypeScript is setup
var useTypeScript = fs_1.default.existsSync(paths_1.default.appTsConfig);
// style files regexes
var cssRegex = /\.css$/;
var cssModuleRegex = /\.module\.css$/;
var sassRegex = /\.(scss|sass)$/;
var sassModuleRegex = /\.module\.(scss|sass)$/;
exports.default = (function (options) {
    var _a = __assign({}, defaultOptions, options), entry = _a.entry, webpackEnv = _a.webpackEnv, htmlFilePathAbs = _a.htmlFilePathAbs, analysis = _a.analysis, servedPath = _a.servedPath, outDir = _a.outDir;
    var isEnvDevelopment = webpackEnv === 'development';
    var isEnvProduction = webpackEnv === 'production';
    var publicPath = isEnvProduction ? servedPath : isEnvDevelopment && '/';
    // common function to get style loaders
    var getStyleLoaders = function (cssOptions, preProcessor) {
        // TODO: 不清楚有什么用
        var shouldUseRelativeAssetPaths = false;
        var loaders = [
            isEnvDevelopment && require.resolve('style-loader'),
            isEnvProduction && {
                loader: mini_css_extract_plugin_1.default.loader,
                options: shouldUseRelativeAssetPaths ? { publicPath: '../../' } : {},
            },
            {
                loader: require.resolve('css-loader'),
                options: cssOptions,
            },
            { loader: 'scoped-css-loader' },
            {
                // Options for PostCSS as we reference these options twice
                // Adds vendor prefixing based on your specified browser support in
                // package.json
                loader: require.resolve('postcss-loader'),
                options: {
                    // Necessary for external CSS imports to work
                    // https://github.com/facebook/create-react-app/issues/2677
                    ident: 'postcss',
                    plugins: function () { return [
                        require('postcss-flexbugs-fixes'),
                        require('postcss-preset-env')({
                            autoprefixer: {
                                flexbox: 'no-2009',
                            },
                            stage: 3,
                        }),
                        // Adds PostCSS Normalize as the reset css with default options,
                        // so that it honors browserslist config in package.json
                        // which in turn let's users customize the target behavior as per their needs.
                        postcss_normalize_1.default(),
                    ]; },
                    sourceMap: isEnvProduction && shouldUseSourceMap,
                },
            },
        ].filter(Boolean);
        if (preProcessor) {
            loaders.push({
                loader: require.resolve(preProcessor),
                options: {
                    sourceMap: isEnvProduction && shouldUseSourceMap,
                },
            });
        }
        return loaders;
    };
    var config = {
        mode: (isEnvProduction
            ? 'production'
            : isEnvDevelopment && 'development'),
        bail: isEnvProduction,
        devtool: (isEnvProduction
            ? shouldUseSourceMap
                ? 'source-map'
                : false
            : isEnvDevelopment && 'cheap-module-source-map'),
        entry: [
            isEnvDevelopment &&
                require.resolve('react-dev-utils/webpackHotDevClient')
        ].concat((Array.isArray(entry) ? entry : [entry])).filter(Boolean),
        output: {
            // The build folder.
            path: isEnvProduction ? utils_1.resolvePathInProject(outDir) : undefined,
            // Add /* filename */ comments to generated require()s in the output.
            pathinfo: isEnvDevelopment,
            // There will be one main bundle, and one file per asynchronous chunk.
            // In development, it does not produce real files.
            // filename: isEnvProduction
            //   ? `${assetsFolder}/js/[name].[contenthash:8].js`
            //   : isEnvDevelopment && `${assetsFolder}/js/bundle.js`,
            // chunkFilename: isEnvProduction
            //   ? `${assetsFolder}/js/[name].[contenthash:8].chunk.js`
            //   : isEnvDevelopment && `${assetsFolder}/js/[name].chunk.js`,
            filename: assetsFolder + "/js/[name].[hash:8].js",
            chunkFilename: assetsFolder + "/js/[name].[hash:8].chunk.js",
            publicPath: publicPath,
        },
        optimization: {
            minimize: isEnvProduction,
            minimizer: [
                // This is only used in production mode
                new terser_webpack_plugin_1.default({
                    terserOptions: {
                        parse: {
                            // we want terser to parse ecma 8 code. However, we don't want it
                            // to apply any minfication steps that turns valid ecma 5 code
                            // into invalid ecma 5 code. This is why the 'compress' and 'output'
                            // sections only apply transformations that are ecma 5 safe
                            // https://github.com/facebook/create-react-app/pull/4234
                            ecma: 8,
                        },
                        compress: {
                            ecma: 5,
                            warnings: false,
                            // Disabled because of an issue with Uglify breaking seemingly valid code:
                            // https://github.com/facebook/create-react-app/issues/2376
                            // Pending further investigation:
                            // https://github.com/mishoo/UglifyJS2/issues/2011
                            comparisons: false,
                            // Disabled because of an issue with Terser breaking valid code:
                            // https://github.com/facebook/create-react-app/issues/5250
                            // Pending futher investigation:
                            // https://github.com/terser-js/terser/issues/120
                            inline: 2,
                        },
                        mangle: {
                            safari10: true,
                        },
                        output: {
                            ecma: 5,
                            comments: false,
                            // Turned on because emoji and regex is not minified properly using default
                            // https://github.com/facebook/create-react-app/issues/2488
                            ascii_only: true,
                        },
                    },
                    // Use multi-process parallel running to improve the build speed
                    // Default number of concurrent runs: os.cpus().length - 1
                    // Disabled on WSL (Windows Subsystem for Linux) due to an issue with Terser
                    // https://github.com/webpack-contrib/terser-webpack-plugin/issues/21
                    parallel: !is_wsl_1.default,
                    // Enable file caching
                    cache: true,
                    sourceMap: shouldUseSourceMap,
                }),
                // This is only used in production mode
                new optimize_css_assets_webpack_plugin_1.default({
                    cssProcessorOptions: {
                        parser: postcss_safe_parser_1.default,
                        map: shouldUseSourceMap
                            ? {
                                // `inline: false` forces the sourcemap to be output into a
                                // separate file
                                inline: false,
                                // `annotation: true` appends the sourceMappingURL to the end of
                                // the css file, helping the browser find the sourcemap
                                annotation: true,
                            }
                            : false,
                    },
                }),
            ],
            // Automatically split vendor and commons
            // https://twitter.com/wSokra/status/969633336732905474
            // https://medium.com/webpack/webpack-4-code-splitting-chunk-graph-and-the-splitchunks-optimization-be739a861366
            splitChunks: {
                chunks: 'all',
                name: false,
            },
            // Keep the runtime chunk separated to enable long term caching
            // https://twitter.com/wSokra/status/969679223278505985
            runtimeChunk: true,
        },
        resolve: {
            extensions: moduleFileExtensions
                .map(function (ext) { return "." + ext; })
                .filter(function (ext) { return useTypeScript || !ext.includes('ts'); }),
            symlinks: false,
        },
        module: {
            strictExportPresence: true,
            rules: [
                // Disable require.ensure as it's not a standard language feature.
                { parser: { requireEnsure: false } },
                // First, run the linter.
                // It's important to do this before Babel processes the JS.
                {
                    test: /\.(js|mjs|jsx|ts|tsx)$/,
                    enforce: 'pre',
                    use: [
                        {
                            options: {
                                formatter: require.resolve('react-dev-utils/eslintFormatter'),
                                eslintPath: require.resolve('eslint'),
                                baseConfig: {
                                    extends: [require.resolve('eslint-config-react-app')],
                                },
                                ignore: false,
                                useEslintrc: false,
                            },
                            loader: require.resolve('eslint-loader'),
                        },
                    ],
                    // TODO: since it may not be src
                    include: paths_1.default.appSrc,
                },
                {
                    // "oneOf" will traverse all following loaders until one will
                    // match the requirements. When no loader matches it will fall
                    // back to the "file" loader at the end of the loader list.
                    oneOf: [
                        // "url" loader works like "file" loader except that it embeds assets
                        // smaller than specified limit in bytes as data URLs to avoid requests.
                        // A missing `test` is equivalent to a match.
                        {
                            test: [/\.bmp$/, /\.gif$/, /\.jpe?g$/, /\.png$/],
                            loader: require.resolve('url-loader'),
                            options: {
                                limit: 10000,
                                name: assetsFolder + "/media/[name].[hash:8].[ext]",
                            },
                        },
                        // Process application JS with Babel.
                        // The preset includes JSX, Flow, TypeScript, and some ESnext features.
                        {
                            test: /\.(js|mjs|jsx|ts|tsx)$/,
                            // TODO: since it may not be src
                            include: paths_1.default.appSrc,
                            loader: require.resolve('babel-loader'),
                            options: {
                                customize: require.resolve('babel-preset-react-app/webpack-overrides'),
                                babelrc: false,
                                configFile: false,
                                presets: [require.resolve('babel-preset-react-app')],
                                // Make sure we have a unique cache identifier, erring on the
                                // side of caution.
                                // We remove this when the user ejects because the default
                                // is sane and uses Babel options. Instead of options, we use
                                // the react-scripts and babel-preset-react-app versions.
                                cacheIdentifier: getCacheIdentifier_1.default(isEnvProduction
                                    ? 'production'
                                    : isEnvDevelopment && 'development', [
                                    'babel-plugin-named-asset-import',
                                    'babel-preset-react-app',
                                    'react-dev-utils',
                                    'react-scripts',
                                ]),
                                plugins: [
                                    require.resolve('babel-plugin-react-scoped-css'),
                                    [
                                        require.resolve('babel-plugin-named-asset-import'),
                                        {
                                            loaderMap: {
                                                svg: {
                                                    ReactComponent: '@svgr/webpack?-svgo,+ref![path]',
                                                },
                                            },
                                        },
                                    ],
                                ],
                                // This is a feature of `babel-loader` for webpack (not Babel itself).
                                // It enables caching results in ./node_modules/.cache/babel-loader/
                                // directory for faster rebuilds.
                                cacheDirectory: true,
                                cacheCompression: isEnvProduction,
                                compact: isEnvProduction,
                            },
                        },
                        // Process any JS outside of the app with Babel.
                        // Unlike the application JS, we only compile the standard ES features.
                        {
                            test: /\.(js|mjs)$/,
                            exclude: /@babel(?:\/|\\{1,2})runtime/,
                            loader: require.resolve('babel-loader'),
                            options: {
                                babelrc: false,
                                configFile: false,
                                compact: false,
                                presets: [
                                    [
                                        require.resolve('babel-preset-react-app/dependencies'),
                                        { helpers: true },
                                    ],
                                ],
                                cacheDirectory: true,
                                cacheCompression: isEnvProduction,
                                cacheIdentifier: getCacheIdentifier_1.default(isEnvProduction
                                    ? 'production'
                                    : isEnvDevelopment && 'development', [
                                    'babel-plugin-named-asset-import',
                                    'babel-preset-react-app',
                                    'react-dev-utils',
                                    'react-scripts',
                                ]),
                                // If an error happens in a package, it's possible to be
                                // because it was compiled. Thus, we don't want the browser
                                // debugger to show the original code. Instead, the code
                                // being evaluated would be much more helpful.
                                sourceMaps: false,
                            },
                        },
                        // "postcss" loader applies autoprefixer to our CSS.
                        // "css" loader resolves paths in CSS and adds assets as dependencies.
                        // "style" loader turns CSS into JS modules that inject <style> tags.
                        // In production, we use MiniCSSExtractPlugin to extract that CSS
                        // to a file, but in development "style" loader enables hot editing
                        // of CSS.
                        // By default we support CSS Modules with the extension .module.css
                        {
                            test: cssRegex,
                            exclude: cssModuleRegex,
                            use: getStyleLoaders({
                                importLoaders: 1,
                                sourceMap: isEnvProduction && shouldUseSourceMap,
                            }),
                            // Don't consider CSS imports dead code even if the
                            // containing package claims to have no side effects.
                            // Remove this when webpack adds a warning or an error for this.
                            // See https://github.com/webpack/webpack/issues/6571
                            sideEffects: true,
                        },
                        // Adds support for CSS Modules (https://github.com/css-modules/css-modules)
                        // using the extension .module.css
                        {
                            test: cssModuleRegex,
                            use: getStyleLoaders({
                                importLoaders: 1,
                                sourceMap: isEnvProduction && shouldUseSourceMap,
                                modules: true,
                                getLocalIdent: getCSSModuleLocalIdent_1.default,
                            }),
                        },
                        // Opt-in support for SASS (using .scss or .sass extensions).
                        // By default we support SASS Modules with the
                        // extensions .module.scss or .module.sass
                        {
                            test: sassRegex,
                            exclude: sassModuleRegex,
                            use: getStyleLoaders({
                                importLoaders: 2,
                                sourceMap: isEnvProduction && shouldUseSourceMap,
                            }, 'sass-loader'),
                            // Don't consider CSS imports dead code even if the
                            // containing package claims to have no side effects.
                            // Remove this when webpack adds a warning or an error for this.
                            // See https://github.com/webpack/webpack/issues/6571
                            sideEffects: true,
                        },
                        // Adds support for CSS Modules, but using SASS
                        // using the extension .module.scss or .module.sass
                        {
                            test: sassModuleRegex,
                            use: getStyleLoaders({
                                importLoaders: 2,
                                sourceMap: isEnvProduction && shouldUseSourceMap,
                                modules: true,
                                getLocalIdent: getCSSModuleLocalIdent_1.default,
                            }, 'sass-loader'),
                        },
                        // "file" loader makes sure those assets get served by WebpackDevServer.
                        // When you `import` an asset, you get its (virtual) filename.
                        // In production, they would get copied to the `build` folder.
                        // This loader doesn't use a "test" so it will catch all modules
                        // that fall through the other loaders.
                        {
                            loader: require.resolve('file-loader'),
                            // Exclude `js` files to keep "css" loader working as it injects
                            // its runtime that would otherwise be processed through "file" loader.
                            // Also exclude `html` and `json` extensions so they get processed
                            // by webpacks internal loaders.
                            exclude: [/\.(js|mjs|jsx|ts|tsx)$/, /\.html$/, /\.json$/],
                            options: {
                                name: assetsFolder + "/media/[name].[hash:8].[ext]",
                            },
                        },
                    ],
                },
            ],
        },
        plugins: [
            htmlFilePathAbs &&
                new html_webpack_plugin_1.default(__assign({ template: htmlFilePathAbs, filename: utils_1.getFilename(htmlFilePathAbs) }, (isEnvProduction && {
                    minify: {
                        removeComments: true,
                        collapseWhitespace: true,
                        removeRedundantAttributes: true,
                        useShortDoctype: true,
                        removeEmptyAttributes: true,
                        removeStyleLinkTypeAttributes: true,
                        keepClosingSlash: true,
                        minifyJS: true,
                        minifyCSS: true,
                        minifyURLs: true,
                    },
                }))),
            // TODO: impl
            // new webpack.DefinePlugin(env.stringified),
            isEnvDevelopment && new webpack_1.default.HotModuleReplacementPlugin(),
            isEnvDevelopment && new case_sensitive_paths_webpack_plugin_1.default(),
            isEnvDevelopment &&
                new WatchMissingNodeModulesPlugin_1.default(paths_1.default.appNodeModules),
            isEnvProduction &&
                new mini_css_extract_plugin_1.default({
                    // Options similar to the same options in webpackOptions.output
                    // both options are optional
                    filename: assetsFolder + "/css/[name].[contenthash:8].css",
                    chunkFilename: assetsFolder + "/css/[name].[contenthash:8].chunk.css",
                }),
            // Generate a manifest file which contains a mapping of all asset filenames
            // to their corresponding output file so that tools can pick it up without
            // having to parse `index.html`.
            new webpack_manifest_plugin_1.default({
                fileName: 'asset-manifest.json',
                publicPath: publicPath,
                generate: function (seed, files) {
                    var manifestFiles = files.reduce(function (manifest, file) {
                        manifest[file.name] = file.path;
                        return manifest;
                    }, seed);
                    return {
                        files: manifestFiles,
                    };
                },
            }),
            // TypeScript type checking
            useTypeScript &&
                new ForkTsCheckerWebpackPlugin_1.default({
                    typescript: resolve_1.default.sync('typescript', {
                        basedir: paths_1.default.appNodeModules,
                    }),
                    async: isEnvDevelopment,
                    useTypescriptIncrementalApi: true,
                    checkSyntacticErrors: true,
                    // TODO: 不知道有什么用
                    // resolveModuleNameModule: process.versions.pnp
                    //   ? `${__dirname}/pnpTs.js`
                    //   : undefined,
                    // resolveTypeReferenceDirectiveModule: process.versions.pnp
                    //   ? `${__dirname}/pnpTs.js`
                    //   : undefined,
                    tsconfig: paths_1.default.appTsConfig,
                    reportFiles: ['**', '!**/__tests__/**', '!**/?(*.)(spec|test).*'],
                    watch: paths_1.default.appSrc,
                    silent: true,
                    // The formatter is invoked directly in WebpackDevServerUtils during development
                    formatter: isEnvProduction ? typescriptFormatter_1.default : undefined,
                }),
            analysis &&
                isEnvProduction &&
                new webpack_bundle_analyzer_1.BundleAnalyzerPlugin({
                    analyzerPort: 8022,
                }),
        ].filter(Boolean),
        // Some libraries import Node modules but don't use them in the browser.
        // Tell Webpack to provide empty mocks for them so importing them works.
        node: {
            module: 'empty',
            dgram: 'empty',
            dns: 'mock',
            fs: 'empty',
            http2: 'empty',
            net: 'empty',
            tls: 'empty',
            child_process: 'empty',
        },
        // TODO: since im not using FileSizeReporter link cra, should this be true?
        // performance: {
        //   hints: 'warning',
        // },
        performance: false,
    };
    return config;
});
//# sourceMappingURL=webpack.config.js.map